try:
    from . import protogen, smgen, umlgen, coggen, vppfs, LanguageCPP, LanguageCsharp
except:
    import protogen, smgen, umlgen, coggen, vppfs, LanguageCPP, LanguageCsharp

import os

''' Generate Entry function for Protocols. 

    The python-interface-generator file used to define the structs etc, is passed in, and called by COG in the second stage.
'''


def Protocol(output_dir, pythoninterfacegeneratorfilename, namespacename, classname, declspec="", author = "", group="", brief="", template_dir=""):
    protogen.Generate(output_dir, pythoninterfacegeneratorfilename, namespacename, classname, declspec, author, group, brief, template_dir)


''' Generate Entry function for State Machines

    COG is not used here...so even though a python-interface-generator file is used by the event structs/parameterization, it needs
    to be called by the callee, and the events interface passed in.
'''


def StateMachine(outputdir, transition_table, eventsinterface, namespacenname, statemachinenameprefix, dclspc="", author="", group="", brief="", templatedir="", __internal="", __copy_other_files=True):
    if not templatedir.strip():
        templatedir = os.path.join(os.path.abspath(os.path.dirname(__file__)), "statemachine_templates_embedded_arm")

    if not os.path.isdir(templatedir):
        print("Error : dir '" + templatedir + "' does not exist. Aborting.")
        return

    language = LanguageCPP.LanguageCPP()
    smgenerator = smgen.CStateMachineGenerator(templatedir, outputdir, eventsinterface, language, author, group, brief)
    if not __internal:
        smgenerator.vpp_filename = "Transition Table"
    else:
        smgenerator.vpp_filename = __internal
    smgenerator.Generate(transition_table, namespacenname, statemachinenameprefix, dclspc, __copy_other_files)


def StateMachine_CSHARP(outputdir, transition_table, eventsinterface, namespacenname, statemachinenameprefix, dclspc="", author="", group="", brief="", templatedir="", __internal="", __copy_other_files=True):
    if not templatedir.strip():
        templatedir = os.path.join(os.path.abspath(os.path.dirname(__file__)), "statemachine_templates_cs_winlinmac")

    if not os.path.isdir(templatedir):
        print("Error : dir '" + templatedir + "' does not exist. Aborting.")
        return

    language = LanguageCsharp.LanguageCsharp()
    smgenerator = smgen.CStateMachineGenerator(templatedir, outputdir, eventsinterface, language, author, group, brief)
    if not __internal:
        smgenerator.vpp_filename = "Transition Table"
    else:
        smgenerator.vpp_filename = __internal
    smgenerator.Generate(transition_table, namespacenname, statemachinenameprefix, dclspc, __copy_other_files)


def StateMachineFromModel(outputdir, vp_project_path, vp_statemachinename, eventsinterface, namespacenname, statemachinenameprefix, dclspc="", author="", group="", brief="", templatedir="", __copy_other_files=True):
    transition_table = vppfs.ExtractTransitionTable(vp_statemachinename, vp_project_path)
    StateMachine(outputdir, transition_table, eventsinterface, namespacenname, statemachinenameprefix, dclspc, author, group, brief, templatedir, os.path.basename(vp_project_path),__copy_other_files)


''' Generate Entry function for Class Diagrams
'''


def UML(outputdir, vp_project_path, vp_classdiagramname, dclspc="", author="", group="", brief="", namespace_to_folders=False, templatefiledir=""):
    language = LanguageCPP.LanguageCPP()
    umlgen.Generate(vp_project_path, vp_classdiagramname, outputdir, language, author, group, brief, namespace_to_folders, dclspc, templatefiledir)

def UML_CSHARP(outputdir, vp_project_path, vp_classdiagramname, dclspc="", author="", group="", brief="", namespace_to_folders=False, templatefiledir=""):
    language = LanguageCsharp.LanguageCsharp()
    umlgen.Generate(vp_project_path, vp_classdiagramname, outputdir, language, author, group, brief, namespace_to_folders, dclspc, templatefiledir)

''' Generate Entry function for template files using COG
'''


def Cogify(output_dir, pythonfile, cog_template_FILEorDIRECTORY, namespacename, classname, author="", group="", brief="", dclspec=""):
    if os.path.isfile(cog_template_FILEorDIRECTORY):
        coggen.GenerateFile(output_dir, pythonfile, cog_template_FILEorDIRECTORY, author, namespacename, classname, group, brief, dclspec)
    else:
        coggen.GenerateDirectory(output_dir, pythonfile, cog_template_FILEorDIRECTORY, author, namespacename, classname, group, brief, dclspec)


def FromTransitionTable(TransitionTable, outputPath):
    assert len(TransitionTable) >= 1
    assert len(TransitionTable[0]) == 5

    plantUml = []

    plantUml.append("@startuml")
    plantUml.append("hide empty description")
    plantUml.append("")
    #plantUml.append("top to bottom direction")
    #plantUml.append("left to right direction")
    #plantUml.append("top to bottom direction")
    #plantUml.append("skinparam nodesep 10")
    plantUml.append("skinparam ranksep 150")
    plantUml.append("")

    initialState = TransitionTable[0][0]
    plantUml.append("[*] --> " + initialState)

    for line in TransitionTable:
        assert len(line) == 5

        Start  = line[0].strip()
        Event  = line[1].strip()
        Guard  = line[4].replace("none", "").replace("None", "")
        Action = line[3].replace("none", "").replace("None", "")
        Next   = line[2].replace("none", "").replace("None", "")
        if not Next:
            Next = Start
        if Guard:
            Guard = "[" + Guard + "]"
        Guard_Action = Guard
        if Action:
            Guard_Action = Guard + "\\n/ " + Action

        plantUml.append(Start + " --> " + Next + " : " + Event + Guard_Action)


    plantUml.append("@enduml")
    with open(outputPath, mode='wt', encoding='utf-8') as myfile:
        myfile.write('\n'.join(plantUml))

def FromSML(fileWithSML, outputPath):
    import re
    lines = []
    with open(fileWithSML, 'r') as f:
        lines = f.readlines()
    assert len(lines) > 0
    has = False
    smltt = []
    tt = []
    ''' Extract only lines for SML TT ... and only the first transition table in case there are multiple'''
    for l in lines:
        s = l.strip()
        if not "//" in s:
            if "state<" in s:
                if not has:
                    has = True
                smltt.append(s)
            else: # Only first set of
                if has:
                    break
    assert len(smltt) > 0
    ''' Format to pythonic TT '''
    for s in smltt:
        # Split START/EVENT/GUARD and ACTION/NEXT
        SEG_AN = s.split("/")
        if len(SEG_AN) != 2:
            raise Exception('Unhandled error #1')
        # Extract Start/Event/Guard
        Start = re.search(r'state<(.*?)>', SEG_AN[0]).group(1)
        Event = re.search(r'event<(.*?)>', SEG_AN[0]).group(1)
        # escape [] chars with backslash as they have special meaning.
        Guard = re.search(r'\[(.*?)\]', SEG_AN[0]).group(1).replace("gnone", "None")
        # Extract Action/Next
        Action = ""
        Next = ""
        if "=" in SEG_AN[1]:
            A_N = SEG_AN[1].split("=")
            Action = A_N[0].strip().replace("none","None")
            Next   = re.search(r'state<(.*?)>', A_N[1]).group(1)
        else:
            Action = SEG_AN[1].strip().replace("none", "None")
        tt.append([Start, Event, Next, Action, Guard])
    assert len(smltt) == len(tt)
    FromTransitionTable(tt, outputPath)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = 'eugene'
'''

    MIT License

    Copyright (c) 2015 Eugene Grobbelaar (email : koh.jaen@yahoo.de)

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.

'''
import time
import datetime
import sys


class Language:


    # Apparently these vars are static.
    Lang = None

    # The last created language is the currently generated one...
    def __init__(self):
        Language.Lang = self

    # ------------------------------ Begin : Language Specifics
    # File extension
    def DotHFile(self):
        raise RuntimeError("Language Feature Not Implemented")

    def DotCPPFile(self):
        raise RuntimeError("Language Feature Not Implemented")

    # Braces
    def OpenBrace(self):
        raise RuntimeError("Language Feature Not Implemented")

    def CloseBrace(self):
        raise RuntimeError("Language Feature Not Implemented")

    # White space
    def WhiteSpace(self, indentationlevels):
        return (indentationlevels+1)*'    '

    # Connection type
    def ConnectionType(self):
        raise RuntimeError("Language Feature Not Implemented")

    def ConnectionTypePtr(self):
        raise RuntimeError("Language Feature Not Implemented")

    # Member access / dereference
    def Accessor(self, is_ptr):
        raise RuntimeError("Language Feature Not Implemented")

    # Bytestream access / dereference
    def BytestreamAccessor(self):
        raise RuntimeError("Language Feature Not Implemented")

    # Byte Stream
    def ByteStreamType(self):
        raise RuntimeError("Language Feature Not Implemented")

    def ByteStreamTypeSharedPtr(self):
        raise RuntimeError("Language Feature Not Implemented")

    def ByteStreamTypeRawPtr(self):
        raise RuntimeError("Language Feature Not Implemented")

    # Convenience
    def MessageDescriptor(self, interface, struct):
        return "" if interface.GetMessageTypeIDStr(struct) == "" else " [ MsgTypeID = " + interface.GetMessageTypeIDStr(struct) + " ]"

    def SharedPtrToType(self, typename):
        raise RuntimeError("Language Feature Not Implemented")

    def PtrToTypeName(self, typename):
        return ""

    def GetFactoryCreateParams(self, struct, interface):
        raise RuntimeError("Language Feature Not Implemented")

    def DeclareStructMembers(self, struct, interface, whitespace, attr_packed=True):
        raise RuntimeError("Language Feature Not Implemented")

    def InstantiateStructMembers(self, struct, interface, whitespace, instancename, accessor):
        raise RuntimeError("Language Feature Not Implemented")

    def RawPtrToType(self, typename):
        raise RuntimeError("Language Feature Not Implemented")

    def TypedefSharedPtrToType(self, typename, newtypename):
        raise RuntimeError("Language Feature Not Implemented")

    def TypedefRawPtrToType(self, typename, newtypename):
        raise RuntimeError("Language Feature Not Implemented")

    # Packedness
    def AddAttributePackedToDecl(self, declaration):
        raise RuntimeError("Language Feature Not Implemented")

    def InstantiatePtrToType(self, typepointername, instancename, typename):
        raise RuntimeError("Language Feature Not Implemented")

    def InstantiateArray(self, instancename, typename, noelements):
        raise RuntimeError("Language Feature Not Implemented")

    def DeleteArray(self, instancename):
        raise RuntimeError("Language Feature Not Implemented")

    def DeletePtrToType(self, instancename):
        raise RuntimeError("Language Feature Not Implemented")

    ''' Instantiate/declare a basic type, with optional initializer. Only declarations can use 'attribute packed' directives.
        use typename = '' to make it a known variable initialization
    '''
    def InstantiateType(self, typename, instancename, initialevalue='', is_attr_packed=False, is_static=False, is_const=False):
        raise RuntimeError("Language Feature Not Implemented")

    # parameters need to be a list of (type, name).
    def DeclareFunction(self, returntype, classname, functionname, is_impl, parameters=None, virtual=False, is_static=False, is_const=False):
        raise RuntimeError("Language Feature Not Implemented")

    def DeclareClass(self, classname, declspec=''):
        raise RuntimeError("Language Feature Not Implemented")

    def ForwareDeclareClass(self, classname):
        raise RuntimeError("Language Feature Not Implemented")

    def DeclareStruct(self, structname, declspec=''):
        raise RuntimeError("Language Feature Not Implemented")

    def DeclareEnum(self, enum):
        raise RuntimeError("Language Feature Not Implemented")

    def DeclareHashDefine(self, name, val):
        raise RuntimeError("Language Feature Not Implemented")

    def DeclareNamespace(self, namespacename):
        raise RuntimeError("Language Feature Not Implemented")

    def UsingNamespace(self, namespacename):
        raise RuntimeError("Language Feature Not Implemented")

    def FormatComment(self, commenttext):
        raise RuntimeError("Language Feature Not Implemented")

    def FormatLongComment(self, commenttext):
        raise RuntimeError("Language Feature Not Implemented")

    # The includes that each generated file needs to have from the provided mini framework
    def LanguageSpecificFrameWorkIncludes(self):
        raise RuntimeError("Language Feature Not Implemented")

    def AddInclude(self, filename):  # for python we can simply replace .h with''
        raise RuntimeError("Language Feature Not Implemented")

    def PrintError(self, message):  # python should split the strings by '<<', as there are more arguments passed this way
        raise RuntimeError("Language Feature Not Implemented")

    def PrintMessage(self, message):
        raise RuntimeError("Language Feature Not Implemented")

    def BooleanTrue(self):
        raise RuntimeError("Language Feature Not Implemented")

    def BooleanFalse(self):
        raise RuntimeError("Language Feature Not Implemented")

    def NullPtr(self):  # should be NULL for C, and None for pooota
        raise RuntimeError("Language Feature Not Implemented")

    def PublicAccessSpecifier(self):
        raise RuntimeError("Language Feature Not Implemented")

    def ProtectedAccessSpecifier(self):
        raise RuntimeError("Language Feature Not Implemented")

    def PrivateAccessSpecifier(self):
        raise RuntimeError("Language Feature Not Implemented")

    def If(self, statement):
        raise RuntimeError("Language Feature Not Implemented")

    def ElseIf(self, statement):
        raise RuntimeError("Language Feature Not Implemented")

    def Else(self):
        raise RuntimeError("Language Feature Not Implemented")

    def This(self):
        raise RuntimeError("Language Feature Not Implemented")

    def ParameterString(self,  parameters=None):
        if parameters is None:
            parameters = []

        if str(type(parameters)) == "<type 'list'>" or str(type(parameters)) == "<class 'list'>":  # Python2 gives the first, python3 the second!
            parameter_string = ''
            size = len(parameters)
            cnt = 0
            for param in parameters:
                parameter_string += param[1]
                if cnt != (size - 1):
                    parameter_string += ', '
                cnt += 1
            return parameter_string

        raise Exception("Please use OrderedDict when passing parameters into 'ParameterString'")

    # 1 means ++
    def For_Range(self, start, stop, incr=1):
        raise RuntimeError("Language Feature Not Implemented")

    def LicenseAgreement(self):
        result = []
        result.append("--------------------------------------------------------------------------------")
        result.append("")
        result.append('    ' + "MIT License")
        result.append("")
        result.append('    ' + "Copyright (c) 2015 Eugene Grobbelaar (email : koh.jaen@yahoo.de)")
        result.append("")
        result.append('    ' + "Permission is hereby granted, free of charge, to any person obtaining a copy")
        result.append('    ' + 'of this software and associated documentation files (the "Software"), to deal')
        result.append('    ' + "in the Software without restriction, including without limitation the rights")
        result.append('    ' + "to use, copy, modify, merge, publish, distribute, sublicense, and/or sell")
        result.append('    ' + "copies of the Software, and to permit persons to whom the Software is")
        result.append('    ' + "furnished to do so, subject to the following conditions:")
        result.append("")
        result.append('    ' + "The above copyright notice and this permission notice shall be included in all")
        result.append('    ' + "copies or substantial portions of the Software.")
        result.append("")
        result.append('    ' + 'THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR')
        result.append('    ' + "IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,")
        result.append('    ' + "FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE")
        result.append('    ' + "AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER")
        result.append('    ' + "LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,")
        result.append('    ' + "OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE")
        result.append('    ' + "SOFTWARE.")
        result.append("")
        result.append("--------------------------------------------------------------------------------")
        result.append('        ' + "This file was generated on    : " + datetime.datetime.fromtimestamp(time.time()).strftime('%Y-%m-%d %H:%M:%S') + ".")
        result.append('        ' + "This file was generated using : " + sys.platform + ".")
        result.append('        ' + "This file was generated by a machine. Do not modify it by hand.")
        result.append("")
        result.append("--------------------------------------------------------------------------------")
        return result

    ''' Additions that were added for class diagram
    '''
    def GetFormatNestedNamespaceBegin(self, classObj):
        raise RuntimeError("Language Feature Not Implemented")

    def GetFormatNestedNamespaceEnd(self, classObj):
        raise RuntimeError("Language Feature Not Implemented")

    def GetFormatClassInheritence(self, classObj, dictOfInheritance):
        raise RuntimeError("Language Feature Not Implemented")

    # Typically what goes into a H file, when the items can NOT be forward declared in H file.
    def GetNotForwardDeclarableHeaderIncludes(self, classObj, namespace_to_folders = False, filter_out_type_not_in_model = False, include_vector_if_needed = False):
        raise RuntimeError("Language Feature Not Implemented")

    # Typically what goes into a CPP file, when the items can be forward declared in H file.
    def GetForwardDeclarableHeaderIncludes(self, classObj, classDiagram):
        raise RuntimeError("Language Feature Not Implemented")

    # Typically what goes into a H file, when the items can be forward declared in H file.
    def GetForwardDeclarations(self, classObj, classDiagram):
        raise RuntimeError("Language Feature Not Implemented")

    # Typically what goes into a H file...to determine the language types to be used for multiplicity
    def GetTypeAndNameFromMultiplicityAndModifier(self, TYPE, TYPE_MODIFIER, MULTIPLICITY, NAME):
        raise RuntimeError("Language Feature Not Implemented")

    # Typically what goes into a H file...Get the declaration string for all attributes of the desired visibility
    def GetAttributeDeclarationsPerVisibility(self, classObj, visibility="all"):
        raise RuntimeError("Language Feature Not Implemented")

    # Typically what goes into a H file...Get the declaration string for all associations of the desired visibility
    def GetAssociationDeclarationsPerVisibility(self, classObj, visibility="all"):
        raise RuntimeError("Language Feature Not Implemented")

    # Will get the declaration string for all attributes of the following visibility
    # - public
    # - protected
    # - private
    # - all (public and protected and private)
    def GetAttributeDeclarationsPerVisibility(self, classObj, visibility="all"):
        raise RuntimeError("Language Feature Not Implemented")

    # Will get the declaration string for all attributes of the following visibility
    # - public
    # - protected
    # - private
    # - all (public and protected and private)
    def GetAssociationDeclarationsPerVisibility(self, classObj, visibility="all"):
        raise RuntimeError("Language Feature Not Implemented")

    # Typically what goes into a CPP file...static declared attributes need to be defined somewhere else.
    def GetStaticAttributeDefinitions(self, classObj):
        raise RuntimeError("Language Feature Not Implemented")

    # Typically what goes into a CPP file...static declared associations need to be defined somewhere else.
    def GetStaticAssociationDefinitions(self, classObj):
        raise RuntimeError("Language Feature Not Implemented")

    def GetOperationPerVisibility(self, classObj, is_impl, visibility="all"):
        raise RuntimeError("Language Feature Not Implemented")

    def GetAttributeAssociationGetterSetter(self, classObj, is_impl=False):
        raise RuntimeError("Language Feature Not Implemented")

    def GetEnumLiterals(self, classObj):
        raise RuntimeError("Language Feature Not Implemented")

    def GetStructMembers(self, classObj):
        raise RuntimeError("Language Feature Not Implemented")

    def GetConstructor(self, classobj, is_impl=False):
        raise RuntimeError("Language Feature Not Implemented")

    def GetProjectIncludes(self, setOfProjectDependencies):
        raise RuntimeError("Language Feature Not Implemented")
    # ------------------------------ End



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = 'eugene'

import os
from collections import OrderedDict
import unittest
import shutil

'''

    MIT License

    Copyright (c) 2015 Eugene Grobbelaar (email : koh.jaen@yahoo.de)

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.

'''
'''
    Preservation works by keeping the same code between two unique tag pairs, in a round trip fashion
    A round-trip is required, so in generation, the 'collect' function should be called before the old generated file is
    overwritten with the newly generated file, and the 'replace' function should be called after the new file has been structured
    and before it is output to disk.

    Any tags no longer present in the new generated file will be lost.

    The TAGs must have a common prefix, with a unique suffix:

    {{{USER_XXX, where XXX is unique throughout the file. This 'XXX' may also be generated in further generations.

    eg:
    {{{USER_MYSTUFF}}}
    // all manual code between these two tags will be preserved between round-trips.
    {{{USER_MYSTUFF}}}

    {{{USER_ is the default prefix tag. It can be replaced with whatever makes writing templates prettier to you.
'''


def CleanUpLine(line):
    return line.replace('\t', '') \
        .replace('\n', '') \
        .replace(r'\t', '') \
        .replace(r'\n', '') \
        .replace(' ', '') \
        .replace('/', '') \
        .replace('*', '') \
        .replace('#', '') \
        .replace('~', '') \
        .replace('`', '') \
        .replace('@', '') \
        .replace('$', '') \
        .replace('%', '') \
        .replace('?', '') \
        .replace('+', '') \
        .replace('}', '') \
        .replace(']', '') \
        .replace('>', '') \
        .replace('=', '')


class Preservative:

    def __init__(self, outputfile_OR_dir):
        self._TAG_PREFIX_ = '{{{USER_'
        # {'FileName', {'Tag', [line1, line2,..., lineX]}}
        self.preserved_tags_per_file = {}
        self.preserved_tags_per_file_WAS_USED = {}

        # Does the input exist? (The input is the output of the code generator before it rewrites it)
        if os.path.exists(outputfile_OR_dir):
            if os.path.isdir(outputfile_OR_dir):
                # Its a directory
                files = os.listdir(outputfile_OR_dir)  # Is the input empty
                if not files:
                    print(outputfile_OR_dir + " is empty, nothing to preserve")
                else:
                    self.Collect(outputfile_OR_dir)
            elif os.path.isfile(outputfile_OR_dir):
                # Its a file
                self.CollectFile(outputfile_OR_dir)
        else:
            print(outputfile_OR_dir + " does not exist, nothing to preserve")

    def CollectFile(self, filename_and_path):
        self.preserved_tags_per_file[filename_and_path] = {}
        self.preserved_tags_per_file_WAS_USED[filename_and_path] = {}
        with open(filename_and_path) as f:
            is_preserving = False
            __current_preservation = None
            __current_preservation_line = ""
            try:  # python 3 gave some sort of decode errors...
                for line in f:
                    tag_found = (line.find(self._TAG_PREFIX_) > -1)
                    if is_preserving and tag_found:
                        # stop
                        self.preserved_tags_per_file[filename_and_path][__current_preservation_line] = []
                        self.preserved_tags_per_file_WAS_USED[filename_and_path][__current_preservation_line] = False
                        for line in __current_preservation:
                            self.preserved_tags_per_file[filename_and_path][__current_preservation_line].append(line)
                        __current_preservation_line = ""
                        is_preserving = False
                        tag_found = False

                    if is_preserving:
                        if __current_preservation is None or __current_preservation_line == "":
                            raise RuntimeError("Empty preservation...something is wrong...")
                        # record
                        __current_preservation.append(line)

                    if not is_preserving and tag_found:
                        # start
                        is_preserving = True
                        __current_preservation_line = CleanUpLine(line)
                        __current_preservation = []
            except UnicodeDecodeError:
                print(
                    "<Warning> UnicodeDecodeError caught on " + filename_and_path + ". If this file is not part of the currently preserved files, then ignore.")
            except:
                print(
                    "<Warning> Unknown error caught on " + filename_and_path + ". If this file is not part of the currently preserved files, then ignore.")

    def Collect(self, directory):
        for root, dirs, files in os.walk(directory):
            for file in files:
                if file.find('.h') > -1 or file.find('.cpp') > -1 or file.find('.py') > -1 or file.find('.cs') > -1:
                    self.CollectFile(os.path.join(root, file))

    # filenames_to_lines is an ordereddictionary {'filename', [line1, line2 ...] ...}
    def Emplace(self, filenames_to_lines):
        for fn, lines in filenames_to_lines.items():
            # if fn in self.preserved_tags_per_file:  # Preferred py2/3 way of testing for a key in a dictionary.
            for outputfile, tags in self.preserved_tags_per_file.items():
                if outputfile.find(fn) > -1:
                    #tags = self.preserved_tags_per_file[fn]
                    new_lines = []
                    tag_found = False
                    tagline = ""
                    for line in lines:
                        new_lines.append(line)
                        cleaned_up_line = CleanUpLine(line)
                        if not tag_found and (cleaned_up_line in tags):
                            tagline = line
                            tag_found = True
                            self.preserved_tags_per_file_WAS_USED[outputfile][cleaned_up_line] = True
                            for i in tags[cleaned_up_line]:
                                new_lines.append(i)
                        elif tag_found and (cleaned_up_line in tags):  # 2nd tag...
                            tag_found = False
                        elif tag_found and not (cleaned_up_line in tags):  # 2nd tag missing...add it
                            new_lines.append(tagline)
                            tag_found = False
                    # replace
                    filenames_to_lines[fn] = new_lines

        for outputfile, tags in self.preserved_tags_per_file_WAS_USED.items():
            for tag, was_used in tags.items():
                if not was_used:
                    if len(self.preserved_tags_per_file[outputfile][tag]) > 0:
                        Lost_Code_TXT_filename = outputfile + ".LostCode.txt"
                        if not Lost_Code_TXT_filename in filenames_to_lines:
                            filenames_to_lines[Lost_Code_TXT_filename] = []
                        filenames_to_lines[Lost_Code_TXT_filename].append(outputfile + "\n")
                        filenames_to_lines[Lost_Code_TXT_filename].append(tag + "\n")
                        for i in self.preserved_tags_per_file[outputfile][tag]:
                            filenames_to_lines[Lost_Code_TXT_filename].append(i + "\n")
                        filenames_to_lines[Lost_Code_TXT_filename].append(tag + "\n")
                        filenames_to_lines[Lost_Code_TXT_filename].append("---------------------------------------------" + "\n")

    # Use this to set a custom code preservation tag prefix
    def SetPrefix(self, new_prefix):
        self._TAG_PREFIX_ = new_prefix


class TestPreservative(unittest.TestCase):
    """ Unit test. Not sure how to programmatically test this.
        Its better done by inspection?
    """
    def setUp(self):
        self.TempFolder = "testpreservative"
        self.HeaderFileName = os.path.join(self.TempFolder, "TestPreservative.h")
        self.SourceFileName = os.path.join(self.TempFolder, "TestPreservative.cpp")
        self.Preserve1 = "aabbccddeeffgghh"
        self.Preserve2 = "1122334455667788"
        self.Preserve3 = "howdydooody"
        self.removeDir()
        self.makeDir()
        self.makeFile(self.HeaderFileName)
        self.makeFile(self.SourceFileName)

    def tearDown(self):
        self.removeDir()

    def removeDir(self):
        if os.path.exists(self.TempFolder):
            try:
                shutil.rmtree(self.TempFolder)
            except OSError:
                print("Deletion of the directory %s failed" % self.TempFolder)

    def makeDir(self):
        try:
            os.mkdir(self.TempFolder)
        except OSError:
            print("Creation of the directory %s failed" % self.TempFolder)

    def makeFile(self, filename):
        with open(filename, 'w+') as temp_file:
            for i in range(10):
                temp_file.write('1' + i*"1" + '\n')
            temp_file.write('{{{USER_STUFF_1\n')
            temp_file.write(self.Preserve1 + '\n')
            temp_file.write(self.Preserve1 + '\n')
            temp_file.write(self.Preserve1 + '\n')
            temp_file.write('{{{USER_STUFF_1\n')
            for i in range(20):
                temp_file.write(10*'2' + i*"2" + '\n')
            temp_file.write('{{{USER_STUFF_2\n')
            temp_file.write(self.Preserve2 + '\n')
            temp_file.write(self.Preserve2 + '\n')
            temp_file.write(self.Preserve2 + '\n')
            temp_file.write('{{{USER_STUFF_2\n')
            for i in range(30):
                temp_file.write(20*'3' + i*"3" + '\n')
            temp_file.write('{{{USER_STUFF_3\n')
            temp_file.write(self.Preserve3 + '\n')
            temp_file.write(self.Preserve3 + '\n')
            temp_file.write(self.Preserve3 + '\n')
            temp_file.write('{{{USER_STUFF_3\n')

    def test_Stuff(self):
        jam = Preservative(self.TempFolder)
        self.newFiles = OrderedDict()
        self.newFiles[self.HeaderFileName] = ['bla bla', '{{{USER_STUFF_2', '{{{USER_STUFF_2', 'crap', 'crap', 'crap', '{{{USER_STUFF_3', '{{{USER_STUFF_3', '{{{USER_STUFF_1', '{{{USER_STUFF_1', 'dang', ]
        self.newFiles[self.SourceFileName] = ['{{{USER_STUFF_3', '{{{USER_STUFF_3', 'bla bla', 'crap', 'crap', 'crap', '{{{USER_STUFF_1', '{{{USER_STUFF_1', '{{{USER_STUFF_2', '{{{USER_STUFF_2', 'dang', ]

        oldHeaderSize = len(self.newFiles[self.HeaderFileName])
        oldSourceSize = len(self.newFiles[self.SourceFileName])

        jam.Emplace(self.newFiles)

        # for each set of tags, the same text should be replaced 3 time...
        self.assertTrue((oldHeaderSize + 9) == len(self.newFiles[self.HeaderFileName]), "Expected at 9 more lines in " + self.HeaderFileName)
        self.assertTrue((oldSourceSize + 9) == len(self.newFiles[self.SourceFileName]), "Expected at 9 more lines in " + self.SourceFileName)

        # Check that the preserved text matches what is expected...
        self.assertIn(self.HeaderFileName, jam.preserved_tags_per_file, "Missing h")
        self.assertIn("{{{USER_STUFF_1", jam.preserved_tags_per_file[self.HeaderFileName], "Missing tag .h(1)")
        self.assertIn("{{{USER_STUFF_2", jam.preserved_tags_per_file[self.HeaderFileName], "Missing tag .h(2)")
        self.assertIn("{{{USER_STUFF_3", jam.preserved_tags_per_file[self.HeaderFileName], "Missing tag .h(3)")
        self.assertEqual(len(jam.preserved_tags_per_file[self.HeaderFileName]["{{{USER_STUFF_1"]), 3, "Wrong size of preserved data .h(1)")
        self.assertEqual(len(jam.preserved_tags_per_file[self.HeaderFileName]["{{{USER_STUFF_2"]), 3, "Wrong size of preserved data .h(2)")
        self.assertEqual(len(jam.preserved_tags_per_file[self.HeaderFileName]["{{{USER_STUFF_3"]), 3, "Wrong size of preserved data .h(3)")
        for line in jam.preserved_tags_per_file[self.HeaderFileName]["{{{USER_STUFF_1"]:
            self.assertTrue(line.find(self.Preserve1) != -1)
        for line in jam.preserved_tags_per_file[self.HeaderFileName]["{{{USER_STUFF_2"]:
            self.assertTrue(line.find(self.Preserve2) != -1)
        for line in jam.preserved_tags_per_file[self.HeaderFileName]["{{{USER_STUFF_3"]:
            self.assertTrue(line.find(self.Preserve3) != -1)

        self.assertIn(self.SourceFileName, jam.preserved_tags_per_file, "Missing cpp")
        self.assertIn("{{{USER_STUFF_1", jam.preserved_tags_per_file[self.SourceFileName], "Missing tag .cpp(1)")
        self.assertIn("{{{USER_STUFF_2", jam.preserved_tags_per_file[self.SourceFileName], "Missing tag .cpp(2)")
        self.assertIn("{{{USER_STUFF_3", jam.preserved_tags_per_file[self.SourceFileName], "Missing tag .cpp(3)")
        self.assertEqual(len(jam.preserved_tags_per_file[self.SourceFileName]["{{{USER_STUFF_1"]), 3, "Wrong size of preserved data .cpp(1)")
        self.assertEqual(len(jam.preserved_tags_per_file[self.SourceFileName]["{{{USER_STUFF_2"]), 3, "Wrong size of preserved data .cpp(2)")
        self.assertEqual(len(jam.preserved_tags_per_file[self.SourceFileName]["{{{USER_STUFF_3"]), 3, "Wrong size of preserved data .cpp(3)")
        for line in jam.preserved_tags_per_file[self.SourceFileName]["{{{USER_STUFF_1"]:
            self.assertTrue(line.find(self.Preserve1) != -1)
        for line in jam.preserved_tags_per_file[self.SourceFileName]["{{{USER_STUFF_2"]:
            self.assertTrue(line.find(self.Preserve2) != -1)
        for line in jam.preserved_tags_per_file[self.SourceFileName]["{{{USER_STUFF_3"]:
            self.assertTrue(line.find(self.Preserve3) != -1)

        # Todo : check that the 'output' has the same stuff? Doing once by inspection is actually ok...however...


def VisualInspect():
    t = TestPreservative()
    t.setUp()
    t.test_Stuff()
    for filename, linesinfile in t.newFiles.items():
        print("****** "+filename+"  ------")
        for line in linesinfile:
            print(line)
    # t.tearDown()


if __name__ == "__main__":
    # VisualInspect()
    unittest.main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = 'eugene'
'''

    MIT License

    Copyright (c) 2015 Eugene Grobbelaar (email : koh.jaen@yahoo.de)

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.

'''
from .Language import *


class LanguagePython(Language):

    # ------------------------------ Begin : Language Specifics
    # File extension
    def DotHFile(self):
        return '.h.py'

    def DotCPPFile(self):
        return '.py'

    # Braces
    def OpenBrace(self):
        return ''

    def CloseBrace(self):
        return ''

    # White space
    def WhiteSpace(self, indentationlevels):
        return (indentationlevels+1)*'    '

    # Connection type
    def ConnectionType(self):
        return 'IConnection'

    def ConnectionTypePtr(self):
        return 'IConnection'

    # Member access / dereference
    def Accessor(self, is_ptr):
        return '.'

    # Bytestream access / dereference
    def BytestreamAccessor(self):
        return '.'

    # Byte Stream
    def ByteStreamType(self):
        return 'bytearray'

    def ByteStreamTypeSharedPtr(self):
        return self.SharedPtrToType(self.ByteStreamType())

    def ByteStreamTypeRawPtr(self):
        return self.RawPtrToType(self.ByteStreamType())

    # Convenience
    def SharedPtrToType(self, typename):
        return typename

    def RawPtrToType(self, typename):
        return typename

    def TypedefSharedPtrToType(self, typename, newtypename):
        return ''

    def TypedefRawPtrToType(self, typename, newtypename):
        return ''

    def GetFactoryCreateParams(self, struct, interface):
        structmembers = struct.Decompose()
        factoryparams = []
        for mem in structmembers:
            isArray = struct.IsArray(mem[1])
            isMessage = interface.IsMessageStruct(mem[0])
            isStruct = struct.IsStruct(mem[1])
            if not interface.IsProtocolStruct(mem[0]):
                if (mem[0] in interface) and not isArray:
                    factoryparams.append(("", mem[1]))
                else:
                    factoryparams.append(("", mem[1]))
        return factoryparams

    '''Declares the guts of a struct declaration
    '''
    def DeclareStructMembers(self, struct, interface, whitespace, attr_packed=True):
        arrayName = []
        structmembers = struct.Decompose()
        result = []
        for mem in structmembers:
            # ptr = ""
            # if struct.IsArray(mem[1]):
            #    ptr = "*"
            if not struct.IsArray(mem[1]):
                result.append(whitespace + self.InstantiateType(mem[0], mem[1]) + '# ' + mem[0])
            else:
                result.append(whitespace + self.InstantiateType(mem[0], mem[1]) + '# ' + mem[0] + '[]')

            if struct.IsArray(mem[1]):
                arrayName.append(mem[1])
        return result

    # Packedness
    def AddAttributePackedToDecl(self, declaration):
        # http://stackoverflow.com/questions/14771150/python-ctypes-pragma-pack-for-byte-aligned-read
        return '_pack_ = 1'

    def InstantiatePtrToType(self, typename, instancename):
        return instancename + ' = ' + typename + '()'

    def InstantiateArray(self, instancename, typename, noelements):
        return instancename + ' = ' + typename + '('+noelements+')\n'

    def DeleteArray(self, instancename):
        return 'del ' + instancename + '\n'

    def DeletePtrToType(self, instancename):
        return 'del ' + instancename + '\n'

    ''' Instantiate/declare a basic type, with optional initializer. Only declarations can use 'attribute packed' directives.
        use typename = '' to make it a known variable initialization
    '''
    def InstantiateType(self, typename, instancename, initialevalue='None', is_attr_packed=False, is_static=False, is_const=False):
        # declaration...i.e. in class or struct. no initial value, can be packed
        # if initialevalue.replace(' ','') == '':
        #    decl =  "('" + instancename + "'," + typename + ")"
        #    return decl + ',\n'

        # local variable decl/instantiation or known variable instantiation
        return instancename + ' = ' + initialevalue

    def InstantiateStructMembers(self, struct, interface, whitespace, instancename, accessor):
        structmembers = struct.Decompose()
        result = []
        for mem in structmembers:
            isArray = struct.IsArray(mem[1])
            isStruct = interface.IsStruct(mem[1])
            isProtocol = interface.IsProtocolStruct(mem[0])
            instance_accessor = whitespace + instancename + accessor

            if not isArray and (not isProtocol or isStruct):
                result.append(instance_accessor + self.InstantiateType('', mem[1], mem[1]))
            elif not isArray and isProtocol and not isStruct:
                s = Template("sizeof(${this}) - sizeof(${header})")
                result.append(instance_accessor + self.InstantiateType("", mem[1], "Create_" + mem[0] + "(" + struct[mem[1]].GetDefaultsAsString(s.substitute(this=struct.Name, header=mem[0])) + ")"))
            elif isArray and not isProtocol and not isStruct:
                if not IsMessageStruct(interface, struct.Name):
                    raise RuntimeError("Only mesage structs are allowed arrays.")
                s = Template("${payload} ${op} ${bytes}")
                array = struct[mem[1]]

                if interface.IsStruct(array.type):
                    result.append(instance_accessor + s.substitute(payload=struct.HeaderName() + "." + struct[struct.HeaderName()].PayloadSize(), op="-=", bytes="sizeof(" + array.type + ")*1"))
                    result.append(instance_accessor + s.substitute(payload=struct.HeaderName() + "." + struct[struct.HeaderName()].PayloadSize(), op="+=", bytes="sizeof(" + array.type + ")*" + array.Count()))
                else:
                    result.append(instance_accessor + s.substitute(payload=struct.HeaderName() + "." + struct[struct.HeaderName()].PayloadSize(), op="-=", bytes="sizeof('" + array.type + "')*1"))
                    result.append(instance_accessor + s.substitute(payload=struct.HeaderName() + "." + struct[struct.HeaderName()].PayloadSize(), op="+=", bytes="sizeof('" + array.type + "')*" + array.Count()))
                # result.append(instance_accessor + self.InstantiateArray(array.type,array.Name,array.Count()))
                # result.append(whitespace + "if(nullptr != " + array.Name + ")")
                result.append(whitespace + instance_accessor.replace("\t", '').replace("    ","") + array.Name + " = " + array.Name)
            else:
                print("WTF : InstantiateStructMembers")

        return result

    # parameters need to be a list of (type, name).
    def DeclareFunction(self, returntype, classname, functionname, is_impl, parameters=None, virtual=False, is_static=False, is_const=False):
        # def DeclareFunction(self, returntype, classname, functionname, is_impl, parameters=[], virtual=False):
        # https://florimond.dev/blog/articles/2018/08/python-mutable-defaults-are-the-source-of-all-evil/
        if parameters is None:
            parameters = []
        if not is_impl:
            return ''

        if str(type(parameters)) == "<type 'list'>" or str(type(parameters)) == "<class 'list'>":  # Python2 gives the first, python3 the second!
            parameter_string = ''

            size = len(parameters)
            cnt = 0
            for param in parameters:
                parameter_string += param[1]
                if cnt != (size - 1):
                    parameter_string += ', '
                cnt += 1

            return 'def ' + functionname + '(' + parameter_string + ')' + ':'

        raise Exception("Please use list when passing parameters into 'DeclareFunction'")

    def DeclareClass(self, classname, declspec=''):
        return 'class ' + classname + ':\n'

    def ForwareDeclareClass(self, classname):
        return ''

    def DeclareStruct(self, structname, declspec=''):
        """c-type structs here
        # http://stackoverflow.com/questions/14771150/python-ctypes-pragma-pack-for-byte-aligned-read
        """
        result = 'class ' + structname + '(Structure):\n'
        result += self.AddAttributePackedToDecl('') + '\n'
        result += '_fields_ = [\n'
        return result

    def DeclareNamespace(self, namespacename):
        return '\n'

    def UsingNamespace(self, namespacename):
        return '\n'

    #def DeclareClassConstructor(self, classname, is_impl, parameters = OrderedDict()):

    def FormatComment(self, commenttext):
        return '# ' + commenttext

    def FormatLongComment(self, commenttext):
        return "'''" + commenttext + "'''\n"

    # The includes that each generated file needs to have from the provided mini framework
    def LanguageSpecificFrameWorkIncludes(self):
        return ['ctypes']

    def AddInclude(self, filename):  # for python we can simply replace .h with''
        return 'from ' + filename.replace('.py', '').replace('"', '').replace('<', '').replace('>', '').replace('\n', '') + ' import *\n'

    def PrintError(self, message):  # python should split the strings by '<<', as there are more arguments passed this way
        return self.PrintMessage("ERROR : " + message)

    def PrintMessage(self, message):
        result = 'print ('
        for i in message.split("<<"):
            result += "+ str(" + i + ")"
        result += ')\n'
        return result

    def BooleanTrue(self):
        return 'True'

    def BooleanFalse(self):
        return 'False'

    def NullPtr(self):  # should be NULL for C, and None for pooota
        return 'None'

    def PublicAccessSpecifier(self):
        return ''

    def ProtectedAccessSpecifier(self):
        return ''

    def PrivateAccessSpecifier(self):
        return ''

    def If(self, statement):
        return 'if '+statement+' :\n'

    def ElseIf(self, statement):
        return 'elif '+statement+' :\n'

    def Else(self):
        return 'else:\n'

    def This(self):
        return 'self.'

    # 1 means ++
    def For_Range(self, start, stop, incr=1):
        if incr == 1:
            return 'for i in range(' + str(start) + ',' + str(stop) + '):'
        return 'for i in range (' + str(start) + ',' + str(stop) + ',' + str(incr) + '):'

    # Python special functions
    def DeclareDataFormatFunction(self, struct, interface, whitespace, formatfunction):
        structmembers = struct.Decompose()
        result = []
        result.append(whitespace + "# Binary data format")
        result.append(whitespace + "@staticmethod")
        hasArray = False
        if interface.IsMessageStruct(struct.Name):
            hasArray = struct.HasArray()
        if not hasArray:
            result.append(whitespace + self.DeclareFunction(struct.Name, "", "BinaryDataFormat", True, []))
        else:
            result.append(whitespace + self.DeclareFunction(struct.Name, "", "BinaryDataFormat", True, [('actual', 'actual')]) + " # Has array, needs actual count.")

        result.append(whitespace + self.WhiteSpace(0) + 'result = ""')
        for mem in structmembers:
            isArray = struct.IsArray(mem[1])
            isStruct = struct.IsStruct(mem[1])
            isProtocol = interface.IsProtocolStruct(mem[0])
            if isStruct or isProtocol:
                result.append(whitespace + self.WhiteSpace(0) + 'result += ' + mem[0] + '.BinaryDataFormat()' + "# " + mem[0] + ' - ' + mem[1])
            elif isArray:
                if interface.IsStruct(mem[0]):
                    result.append(whitespace + self.WhiteSpace(0) + 'for i in range(actual.' + mem[1] + '_Cnt):' + "# " + mem[0] + ' - ' + mem[1])
                    result.append(whitespace + self.WhiteSpace(1) + 'result += ' + mem[0] + '.BinaryDataFormat()')
                else:
                    result.append(whitespace + self.WhiteSpace(0) + 'result += str(actual.' + mem[1] + "_Cnt) + " + formatfunction + '("' + mem[0] + '")' + "# " + mem[0] + ' - ' + mem[1])
            else:
                result.append(whitespace + self.WhiteSpace(0) + 'result += '+formatfunction+'("' + mem[0] + '")' + "# " + mem[0] + ' - ' + mem[1])
        result.append(whitespace + self.WhiteSpace(0) + 'return "="+result.replace("=","") # "=" for no padding...only need the first one, this removes nested "="\n')
        return result

    def DeclareDataSizeFunction(self, struct, interface, whitespace):
        result = []

        hasArray = False
        if interface.IsMessageStruct(struct.Name):
            hasArray = struct.HasArray()

        structmembers = struct.Decompose()

        result.append(whitespace + "# Binary data size")
        result.append(whitespace + "@staticmethod")
        if hasArray:
            result.append(whitespace + self.DeclareFunction(struct.Name, '', "SizeOf", True, [('actual = None', 'actual = None')]))
            result.append(whitespace + self.WhiteSpace(0) + 'if None == actual: # Factory create will calculate correctly')
            result.append(whitespace + self.WhiteSpace(1) + self.InstantiatePtrToType(struct.Name, 'actual'))
            for mem in structmembers:
                if mem[1].find('_Cnt') > -1:
                    result.append(whitespace + self.WhiteSpace(1) + 'actual.' + mem[1] + ' = 1')
            result.append(whitespace + self.WhiteSpace(0) + 'return calcsize(' + struct.Name + '.BinaryDataFormat(actual))\n')
        else:
            result.append(whitespace + self.DeclareFunction(struct.Name, '', "SizeOf", True, []))
            result.append(whitespace + self.WhiteSpace(0) + 'return calcsize(' + struct.Name + '.BinaryDataFormat())\n')
        return result

    # ------------------------------ End



import setuptools

with open("README.MD", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="kojen",
    version="1.1.23",
    author="kohjaen",
    author_email="koh.jaen@yahoo.de",
    description="Code generation tools.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/kohjaen/kojen",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6.8',
	package_data={'': ['allplatforms/CPP/*.*', 
                       'allplatforms/CPP/sml/include/boost/*.*', 
                       'allplatforms/CPP/testsuite/*.*', 
                       'allplatforms/CPP/testsuite/minunit/minunit.h', 
                       'allplatforms/CPP/testsuite/minunit/minunit.cpp', 
                       'classdiagram_templates/C#/*.*', 
                       'classdiagram_templates/CPP/*.*', 
                       'protocol_templates/CPP/*.*', 
                       'protocol_templates/PY/*.*',
                       'statemachine_templates_embedded_arm/*.*',
                       'statemachine_templates_pc_boost/*.*', 
                       'docs/images/*.*', 
                       'docs/*.*']},
	include_package_data=True,
    install_requires=['cogapp>=3.0.0',],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = 'eugene'
'''

    MIT License

    Copyright (c) 2015 Eugene Grobbelaar (email : koh.jaen@yahoo.de)

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.

'''
import os, shutil
import fnmatch
import sys

# Python2 -> 3 shennanigans...try support both
try:
    from smgen import *		# py2
    from cgen import FileCopyUtil
except (ModuleNotFoundError, ImportError) as e:
    from .smgen import *	# py3
    from .cgen import FileCopyUtil

from cogapp import *

def GetDir(rootdir, dir):
    matches = []
    for root, dirnames, filenames in os.walk(rootdir):
        for filename in fnmatch.filter(dirnames, dir):
            matches.append(os.path.join(root, filename))
    return matches


def GetFile(rootdir, file):
    matches = []
    for root, dirnames, filenames in os.walk(rootdir):
        for filename in fnmatch.filter(filenames, file):
            matches.append(os.path.join(root, filename))


def Generate(output_dir, pythonfile, namespacename, classname, declspec="", author="", group="", brief="", template_dir=""):

    if not template_dir.strip():
        template_dir = os.path.join(os.path.abspath(os.path.dirname(__file__)), os.path.join("protocol_templates", "CPP"))

    if not os.path.isdir(template_dir):
        print("Error : dir '" + template_dir + "' does not exist. Aborting.")
        return

    print("*************************************")
    print("******* Protogen ********************")
    print("*************************************")
    print(" Template Dir : " + template_dir)
    print(" Output Dir   : " + output_dir)
    print(" Python File  : " + pythonfile)
    print(" Executing in : " + os.path.realpath(__file__))
    print("*************************************")

    # Use the directory THIS file is in as the intermediate path
    intermediate_dir = os.path.dirname(os.path.realpath(__file__))

    # copy the python file to this directory...
    Tmp_py = os.path.basename(pythonfile)
    tmp_python_file = os.path.join(intermediate_dir, Tmp_py)
    shutil.copy(pythonfile,tmp_python_file)

    # Customize the templates with your interface file, namespace names, classnames, declspecs
    smgen = CStateMachineGenerator(template_dir, intermediate_dir, None, None, author, group, brief)
    filenames = smgen.GenerateProtocol(Tmp_py, namespacename, classname, declspec, output_dir)

    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    # Create Cog Input
    writer = open(os.path.join(intermediate_dir, "files.txt"), 'w')
    for f in filenames:
        writer.write('"' + os.path.abspath(os.path.join(intermediate_dir, f)) + '" -o "' + os.path.abspath(os.path.join(output_dir, f)) + '"\r\n')
    writer.close()

    '''
    # Invoke cog on intermediate dir
    '''

    c = Cog()
    c.callableMain(['cog', '-d', '@' + os.path.join(intermediate_dir, "files.txt")])

    # Delete all files from intermediate dir
    os.remove(os.path.join(intermediate_dir, "files.txt"))
    for f in filenames:
        os.remove(os.path.join(intermediate_dir, f))
    os.remove(tmp_python_file)

    # Copy non-autogenerated required files to output.
    # Crude : which ones to use? Currently only exist for CPP.
    if template_dir.lower().find("cpp") > -1: # cpp
        # Files...
        files_to_copy = []
        files_to_copy.append("allocator.h")
        files_to_copy.append("allocator.cpp")
        files_to_copy.append("basetypes.h")
        files_to_copy.append("CMakeLists.txt")
        files_to_copy.append("IConnection.h")
        files_to_copy.append("IConnection.cpp")
        files_to_copy.append("IMsgReceiver.h")
        files_to_copy.append("IRawDataReceiver.h")
        files_to_copy.append("MsgHeader.h")
        files_to_copy.append("network.h")
        files_to_copy.append("network.cpp")
        files_to_copy.append("serialport.h")
        files_to_copy.append("serialport.cpp")
        files_to_copy.append("tcpipclientserver.h")
        files_to_copy.append("tcpipclientserver.h")

        allplatformsfrom = os.path.join(os.path.abspath(os.path.dirname(__file__)), os.path.join("allplatforms","CPP"))
        allplatformsto = os.path.join(os.path.abspath(output_dir), "allplatforms")

        FileCopyUtil(allplatformsfrom,allplatformsto,files_to_copy)

        # Tests...
        testfiles_to_copy = []
        testfiles_to_copy.append("CMakeLists.txt")
        testfiles_to_copy.append("Test.IConnection.cpp")
        testfiles_to_copy.append("test_main.cpp")

        tests_allplatformsfrom = os.path.join(allplatformsfrom, "testsuite")
        tests_allplatformsto = os.path.join(allplatformsto, "testsuite")

        FileCopyUtil(tests_allplatformsfrom, tests_allplatformsto, testfiles_to_copy)

        # Micro Unit Test Framework
        microunit_files_to_copy = []
        microunit_files_to_copy.append("minunit.h")
        microunit_files_to_copy.append("minunit.cpp")

        microunit_allplatformsfrom = os.path.join(tests_allplatformsfrom, "minunit")
        microunit_allplatformsto = os.path.join(tests_allplatformsto, "minunit")

        FileCopyUtil(microunit_allplatformsfrom, microunit_allplatformsto, microunit_files_to_copy)


if __name__ == "__main__":
    import sys
    if len(sys.argv) < 5:
        print("Usage : python protogen <output dir> <python if file> <namespace> <class> optional <declspec> optional <template dir>")
    else:
        output_dir = str(sys.argv[1])
        pythonfile = str(sys.argv[2])
        namespacename = str(sys.argv[3])
        classname = str(sys.argv[4])
        declspec = ""
        if len(sys.argv) >= 6:
            declspec = str(sys.argv[5])
        template_dir = ""
        if len(sys.argv) >= 7:
            template_dir = str(sys.argv[6])

        Generate(output_dir, pythonfile, namespacename, classname, declspec, template_dir)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = 'eugene'

'''

    MIT License

    Copyright (c) 2015 Eugene Grobbelaar (email : koh.jaen@yahoo.de)

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.

'''
__TAG_AUTHOR__                      = '<<<AUTHOR>>>'
__TAG_GROUP__                       = '<<<GROUP>>>'
__TAG_BRIEF__                       = '<<<BRIEF>>>'
__TAG_NAMESPACE__                   = '<<<NAMESPACE>>>'
__TAG_SM_NAME__                     = '<<<STATEMACHINENAME>>>'       # As given
__TAG_SM_NAME_SMALL_CAMEL__         = '<<<stateMachineName>>>'       # camelCaps
__TAG_SM_NAME_UPPER__               = '<<<STATEMACHINENAMEUPPER>>>'  # ALL UPPER
__TAG_CLASS_NAME__                  = '<<<CLASSNAME>>>'
__TAG_PyIFGen_NAME__                = '<<<PYIFGENNAME>>>'

__TAG_PS_BEGIN__                    = "<<<PER_STATE_BEGIN>>>"
__TAG_PS_END__                      = "<<<PER_STATE_END>>>"

__TAG_PST_BEGIN__                   = "<<<PER_STATETRANSITION_BEGIN>>>"
__TAG_PST_END__                     = "<<<PER_STATETRANSITION_END>>>"
__TAG_PET_BEGIN__                   = "<<<PER_EVENTTRANSITION_BEGIN>>>"
__TAG_PET_END__                     = "<<<PER_EVENTTRANSITION_END>>>"
__TAG_PGT_BEGIN__                   = "<<<PER_GUARDTRANSITION_BEGIN>>>"
__TAG_PGT_END__                     = "<<<PER_GUARDTRANSITION_END>>>"

__TAG_PE_BEGIN__                    = "<<<PER_EVENT_BEGIN>>>"
__TAG_PE_END__                      = "<<<PER_EVENT_END>>>"

__TAG_PA_BEGIN__                    = "<<<PER_ACTION_BEGIN>>>"
__TAG_PA_END__                      = "<<<PER_ACTION_END>>>"

__TAG_PASIG_BEGIN__                 = "<<<PER_ACTION_SIGNATURE_BEGIN>>>"
__TAG_PASIG_END__                   = "<<<PER_ACTION_SIGNATURE_END>>>"

__TAG_PG_BEGIN__                    = "<<<PER_GUARD_BEGIN>>>"
__TAG_PG_END__                      = "<<<PER_GUARD_END>>>"

__TAG_EVENT_SIGNATURE__             = "<<<EVENTSIGNATURE>>>"
__TAG_EVENT_MEMBERINST__            = "<<<EVENTMEMBERSINSTANTIATE>>>"
__TAG_LITE_EVENT_MEMBERINST__       = "<<<EVENTMEMBERSLITEINSTANTIATE>>>"
__TAG_EVENT_MEMBERDECL__            = "<<<EVENTMEMBERSDECLARE>>>"

__TAG_STATENAME__                   = '<<<STATENAME>>>'            # As given
__TAG_STATENAME_SMALL_CAMEL__       = '<<<stateName>>>'            # camelCaps
__TAG_NEXTSTATENAME__               = '<<<NEXTSTATENAME>>>'        # As given
__TAG_NEXTSTATENAME_SMALL_CAMEL__   = '<<<nextStateName>>>'        # camelCaps
__TAG_STATENAME_IF_NEXTSTATE__      = '<<<STATENAMEIFNEXTSTATE>>>' # As given
__TAG_STATENAME_IF_NEXTSTATE_SMALL_CAMEL__      = '<<<stateNameIfNextState>>>' # As given
__TAG_EVENTNAME__                   = '<<<EVENTNAME>>>'            # As given
__TAG_EVENTNAME_SMALL_CAMEL__       = '<<<eventName>>>'            # camelCaps
__TAG_ACTIONNAME__                  = '<<<ACTIONNAME>>>'           # As given
__TAG_ACTIONNAME_SMALL_CAMEL__      = '<<<actionName>>>'           # camelCaps
__TAG_GUARDNAME__                   = '<<<GUARDNAME>>>'            # As given
__TAG_GUARDNAME_SMALL_CAMEL__       = '<<<guardName>>>'            # camelCaps

__TAG_ABC__                         = '<<<ALPH>>>'
__TAG_123__                         = '<<<NUM>>>'
__TAG_INIT_STATE__                  = '<<<STATE_0>>>'              # As given
__TAG_INIT_STATE_SMALL_CAMEL__      = '<<<state_0>>>'              # camelCaps

__TAG_TTT_BEGIN__                   = '<<<TTT_BEGIN>>>'
__TAG_TTT_END___                    = '<<<TTT_END>>>'

__TAG_TTT_LITE_BEGIN__              = '<<<TTT_LITE_BEGIN>>>'
__TAG_TTT_LITE_END__                = '<<<TTT_LITE_END>>>'

__TAG_TTT_SML_BEGIN__               = '<<<TTT_SML_BEGIN>>>'
__TAG_TTT_SML_BEGIN_ENTRYEXIT__     = '<<<TTT_SML_BEGIN_ENTRYEXIT>>>'
__TAG_TTT_SML_END__                 = '<<<TTT_SML_END>>>'

__TAG_DECLSPEC_DLL_EXPORT__         = "<<<DLL_EXPORT>>>"

# Python2 -> 3 shennanigans...try support both
try:
    from kojentypes import *		# py2
except (ModuleNotFoundError, ImportError) as e:
    from .kojentypes import *		# py3

try:
    from .preservative import *
except (ModuleNotFoundError, ImportError) as e:
    from preservative import *

try:
    from .cgen import CBASEGenerator, CCodeModel, alpha, __getnextalphabet__, __resetalphabet__, even_space, FileCopyUtil, caps, camel_case_small, camel_case
except (ModuleNotFoundError, ImportError) as e:
    from cgen import CBASEGenerator, CCodeModel, alpha, __getnextalphabet__, __resetalphabet__, even_space, FileCopyUtil, caps, camel_case_small, camel_case

try:
    from LanguageCPP import LanguageCPP
except  (ModuleNotFoundError, ImportError) as e:
    from .LanguageCPP import LanguageCPP

# Model that describes a state machine.
class CStateMachineModel:
    def __init__(self):
        self.statemachinename = ""
        self.namespacename    = ""
        self.declspecdllexport = ""
        self.pythoninterfacegeneratorfilename = ""
        self.states           = set()
        self.actions          = set()
        self.events           = set()
        self.guards           = set()
        self.actionsignatures = OrderedDict()
        self.transitionsperstate = OrderedDict()

# Transition Table Model uses State Machine Model to generate all code required for a working state machine.
class CTransitionTableModel(CStateMachineModel):
    START_STATE = 0
    EVENT       = 1
    NEXT_STATE  = 2
    ACTION      = 3
    GUARD       = 4

    def __init__(self, tt, nn, smn, dclspc = ""):
        CStateMachineModel.__init__(self)
        self.transition_table   = tt
        self.statemachinename   = smn
        self.namespacename      = nn
        self.declspecdllexport  = dclspc
        tstate  = OrderedDict()
        taction = OrderedDict()
        tevent  = OrderedDict()
        tguard  = OrderedDict()
        self.maxlenSTART_STATE = 0
        self.maxlenEVENT = 0
        self.maxlenACTION = 0
        self.maxlenGUARD = 0

        length = 0
        # Filter
        for tableline in self.transition_table:
            if tableline[self.START_STATE] != "" and tableline[self.START_STATE].lower() != "none":
                tstate[tableline[self.START_STATE]] = 0
                length = len(tableline[self.START_STATE])
                if self.maxlenSTART_STATE < length:
                    self.maxlenSTART_STATE = length
            if tableline[self.NEXT_STATE] != "" and tableline[self.NEXT_STATE].lower() != "none":
                tstate[tableline[self.NEXT_STATE]] = 0
            if tableline[self.EVENT] != "" and tableline[self.EVENT].lower() != "none":
                tevent[tableline[self.EVENT]] = 0
                length = len(tableline[self.EVENT])
                if self.maxlenEVENT < length:
                    self.maxlenEVENT = length
            if tableline[self.ACTION] != "" and tableline[self.ACTION].lower() != "none":
                taction[tableline[self.ACTION]] = 0
                length = len(tableline[self.ACTION])
                if self.maxlenACTION < length:
                    self.maxlenACTION = length
                if not ((tableline[self.ACTION] + tableline[self.EVENT]) in self.actionsignatures):
                    self.actionsignatures[tableline[self.ACTION] + tableline[self.EVENT]] = (tableline[self.ACTION], tableline[self.EVENT])  #, tableline[self.START_STATE],tableline[self.NEXT_STATE]))
            if tableline[self.GUARD] != "" and tableline[self.GUARD].lower() != "none":
                tguard[tableline[self.GUARD]] = 0
                length = len(tableline[self.GUARD])
                if self.maxlenGUARD < length:
                    self.maxlenGUARD = length
        # Populate CStateMachineModel
        for s in tstate:
            self.states.add(s)
        for e in tevent:
            self.events.add(e)
        for a in taction:
            self.actions.add(a)
        for g in tguard:
            self.guards.add(g)
        self.set_transitions_per_state()

    ''' Returns a dictionary of dictionaries of lists of dictionaries.
    
        {'StateName': {'EventName',[{'GuardName':'', 'ActionName':'val'},{}...]}}
        First dictionary key is the 'state'.
        Second dictionary is the 'event' name.
        These contain a list of dictionaries, as multiple of the same events can
        cause different transitions based on guards.
    '''
    def set_transitions_per_state(self):
        for tableline in self.transition_table:
            transition = OrderedDict()
            if tableline[self.ACTION] != "" and tableline[self.ACTION].lower() != "none":
                transition[__TAG_ACTIONNAME__] = tableline[self.ACTION]
                transition[__TAG_ACTIONNAME_SMALL_CAMEL__] = camel_case_small(tableline[self.ACTION])
            if tableline[self.GUARD] != "" and tableline[self.GUARD].lower() != "none":
                transition[__TAG_GUARDNAME__] = tableline[self.GUARD]
                transition[__TAG_GUARDNAME_SMALL_CAMEL__] = camel_case_small(tableline[self.GUARD])
            if tableline[self.NEXT_STATE] != "" and tableline[self.NEXT_STATE].lower() != "none":
                transition[__TAG_STATENAME_IF_NEXTSTATE__] = tableline[self.START_STATE]
                transition[__TAG_STATENAME_IF_NEXTSTATE_SMALL_CAMEL__] = camel_case_small(tableline[self.START_STATE])
                transition[__TAG_NEXTSTATENAME__] = tableline[self.NEXT_STATE]
                transition[__TAG_NEXTSTATENAME_SMALL_CAMEL__] = camel_case_small(tableline[self.NEXT_STATE])

            if tableline[self.START_STATE] != "" and tableline[self.START_STATE].lower() != "none":
                if not tableline[self.START_STATE] in self.transitionsperstate:
                    self.transitionsperstate[tableline[self.START_STATE]] = OrderedDict()

            if tableline[self.EVENT] != "" and tableline[self.EVENT].lower() != "none":
                if not tableline[self.EVENT] in self.transitionsperstate[tableline[self.START_STATE]]:
                    self.transitionsperstate[tableline[self.START_STATE]][tableline[self.EVENT]] = []
                self.transitionsperstate[tableline[self.START_STATE]][tableline[self.EVENT]].append(transition)

    def getfirststate(self):
        if not self.transition_table:
            return "NO TT PRESENT!"
        return self.transition_table[0][0]


class CStateMachineGenerator(CBASEGenerator):

    def __init__(self, inputfiledir, outputfiledir, events_interface=None, language=None, author='Anonymous', group='', brief=''):
        CBASEGenerator.__init__(self,inputfiledir,outputfiledir,language, author, group, brief)
        self.events_interface = events_interface
        self.vpp_filename = ""

    def loadtemplates_firstfiltering(self, smmodel):
        """
        See baseclass implementation. This just prepares the dictionary of things to replace
        for this type of codegeneration.

        @param smmodel:
        @return: cgen.CCodeModel, a dictionary -> {filename,[lines]}
        """

        dict_to_replace_lines = {}
        dict_to_replace_lines[__TAG_SM_NAME_UPPER__] = caps(smmodel.statemachinename)
        dict_to_replace_lines[__TAG_SM_NAME_SMALL_CAMEL__] = camel_case_small(smmodel.statemachinename)
        dict_to_replace_lines[__TAG_SM_NAME__] = smmodel.statemachinename
        dict_to_replace_lines[__TAG_CLASS_NAME__] = smmodel.statemachinename
        dict_to_replace_lines[__TAG_PyIFGen_NAME__] = smmodel.pythoninterfacegeneratorfilename.replace('.py', '')  # hack : for tcpgen simple templates,
        if not dict_to_replace_lines[__TAG_PyIFGen_NAME__]:
            dict_to_replace_lines[__TAG_PyIFGen_NAME__] = self.vpp_filename
        dict_to_replace_lines[__TAG_NAMESPACE__] = smmodel.namespacename
        dict_to_replace_lines[__TAG_AUTHOR__] = self.author
        dict_to_replace_lines[__TAG_GROUP__] = self.group
        dict_to_replace_lines[__TAG_BRIEF__] = self.brief
        dict_to_replace_lines[__TAG_DECLSPEC_DLL_EXPORT__] = smmodel.declspecdllexport

        dict_to_replace_filenames = {}
        dict_to_replace_filenames["TEMPLATE_"] = smmodel.statemachinename

        return CBASEGenerator.loadtemplates_firstfiltering(self,dict_to_replace_lines,dict_to_replace_filenames)

    def get_event_signature(self,name):
        if self.events_interface is None or self.language is None:
            return ""
        for s in self.events_interface.Structs():
            if s.Name == name:
                return self.language.ParameterString(self.language.GetFactoryCreateParams(s, self.events_interface))

        return ""

    def instantiate_event_struct_member(self, name, whitespace_cnt, is_ptr=True, instancename="data"):
        if self.events_interface is None or self.language is None:
            return ""
        for s in self.events_interface.Structs():
            if s.Name == name:
                guts = self.language.InstantiateStructMembers(s, self.events_interface, '', instancename, self.language.Accessor(is_ptr))
                result = ''
                cnt = 0
                for g in guts:
                    result = result + (whitespace_cnt*'    ' if cnt > 0 else '') + g + '\n'
                    cnt = cnt + 1
                return result.rsplit('\n', 1)[0]
        return ""

    def declare_event_struct_members(self, name, whitespace_cnt):
        if self.events_interface is None or self.language is None:
            return ""

        for s in self.events_interface.Structs():
            if s.Name == name:
                guts = self.language.DeclareStructMembers(s, self.events_interface, '', False)
                result = ''
                cnt = 0
                for g in guts:
                    result = result + ((whitespace_cnt+1)*'    ' if cnt > 0 else '    ') + g + '\n'
                    cnt = cnt + 1
                return result.rsplit('\n', 1)[0]
        return ""

    def innerexpand_secondfiltering(self, names2x, lines2x, puthere):
        global alpha
        __resetalphabet__()
        cnt = 0
        for name in names2x:
            for line in lines2x:
                newline = line
                newline = newline.replace(__TAG_STATENAME_SMALL_CAMEL__, camel_case_small(name))
                newline = newline.replace(__TAG_STATENAME__, name)
                newline = newline.replace(__TAG_EVENTNAME_SMALL_CAMEL__, camel_case_small(name))
                newline = newline.replace(__TAG_EVENTNAME__, name)
                newline = newline.replace(__TAG_ACTIONNAME__, name)
                newline = newline.replace(__TAG_ACTIONNAME_SMALL_CAMEL__, camel_case_small(name))
                newline = newline.replace(__TAG_GUARDNAME__, name)
                newline = newline.replace(__TAG_GUARDNAME_SMALL_CAMEL__, camel_case_small(name))
                newline = newline.replace(__TAG_ABC__, chr(alpha))
                newline = newline.replace(__TAG_123__, str(cnt))
                tabcnt = newline.count('    ')
                if self.hasSpecificTag(newline,__TAG_EVENT_SIGNATURE__):
                    if self.hasDefault(newline):
                        # Has user params
                        user_params = self.extractDefaultAndTag(newline)
                        signature = self.get_event_signature(name) + ", " + user_params[1]
                        signature = signature.strip(',').strip(' ')
                        newline = newline.replace(user_params[0], signature)
                    else:
                        newline = newline.replace(__TAG_EVENT_SIGNATURE__, self.get_event_signature(name))
                # __TAG_EVENT_MEMBERINST__ -> PTR
                if self.hasSpecificTag(newline,__TAG_EVENT_MEMBERINST__) and self.hasDefault(newline):
                    line_member = self.extractDefaultAndTag(newline)
                    newline = newline.replace(line_member[0],self.instantiate_event_struct_member(name, tabcnt, True, line_member[1]))
                else:
                    newline = newline.replace(__TAG_EVENT_MEMBERINST__, self.instantiate_event_struct_member(name, tabcnt, True))        # PTR
                # __TAG_LITE_EVENT_MEMBERINST__ -> NO PTR
                if self.hasSpecificTag(newline,__TAG_LITE_EVENT_MEMBERINST__) and self.hasDefault(newline):
                    line_member = self.extractDefaultAndTag(newline)
                    newline = newline.replace(line_member[0],self.instantiate_event_struct_member(name, tabcnt, False, line_member[1]))
                else:
                    newline = newline.replace(__TAG_LITE_EVENT_MEMBERINST__, self.instantiate_event_struct_member(name, tabcnt, False))  # NO PTR
                newline = newline.replace(__TAG_EVENT_MEMBERDECL__, self.declare_event_struct_members(name, tabcnt))
                if newline == '\n' or newline == '' or newline == '\r\n' or newline.replace(' ','') == "" or newline.replace(' ','').replace('\n','').replace('\r','') == "":
                    continue
                puthere.append(newline)
            cnt = cnt + 1
            __getnextalphabet__()

    def innerexpand_actionsignatures(self, states2x, lines2x, puthere):
        global alpha
        __resetalphabet__()
        cnt = 0
        for key, (actionname, eventname) in states2x.items():
            if eventname == "" or eventname.lower() == 'none':
                eventname = "NONE"
            elif eventname.lower() == 'any':
                eventname = "ANY"
            for line in lines2x:
                puthere.append(line
                            .replace(__TAG_ACTIONNAME_SMALL_CAMEL__, camel_case_small(actionname))
                            .replace(__TAG_ACTIONNAME__, actionname)
                            .replace(__TAG_EVENTNAME_SMALL_CAMEL__, camel_case_small(eventname))
                            .replace(__TAG_EVENTNAME__, eventname)
                            .replace(__TAG_ABC__, chr(alpha))
                            .replace(__TAG_123__, str(cnt)))
            cnt = cnt + 1
            __getnextalphabet__()

    def innerexpand_msm(self, smmodel, puthere):
        len_tt = len(smmodel.transition_table)
        tt_out = "        // " + len("msmf::Row < ") * ' ' + even_space("Start") + even_space("Event") + even_space("Next") + even_space("Action") + even_space("Guard") + '\n'
        for i, ttline in enumerate(smmodel.transition_table):
            tt_out += '        msmf::Row < '
            tt_out += even_space(self.transitiontable_replace_NONE(ttline[smmodel.START_STATE])) + ','
            tt_out += even_space(self.transitiontable_replace_NONE(ttline[smmodel.EVENT])) + ','
            tt_out += even_space(self.transitiontable_replace_NONE(ttline[smmodel.NEXT_STATE])) + ','
            tt_out += even_space(self.transitiontable_replace_NONE(ttline[smmodel.ACTION])) + ','
            tt_out += even_space(self.transitiontable_replace_NONE(ttline[smmodel.GUARD])) + '>    '
            if i != len_tt - 1:
                tt_out += ","
            tt_out += "    // " + str(i) + '\n'
            puthere.append(tt_out)
            tt_out = ""

    def innerexpand_msmlite(self, smmodel, puthere):
        tt_out = "                // " + even_space("Start + ") + even_space("Event") + even_space("[ Guard ] ") + even_space("/ Action") + even_space(" = Next") + '\n'
        startStateHasEntryExit = {}
        for i, ttline in enumerate(smmodel.transition_table):
            if i == 0:  # initial state
                tt_out += "                 *"
            else:
                tt_out += "                , "
            tt_out += even_space(self.transitiontable_replace_NONE(ttline[smmodel.START_STATE])) + '+'
            tt_out += even_space('event<' + self.transitiontable_replace_NONE(ttline[smmodel.EVENT]) + ">") + ' '
            tt_out += even_space('[' + self.transitiontableLITE_guard_replace_NONE('__' + ttline[smmodel.GUARD]) + ']') + ' / '
            tt_out += even_space(self.transitiontableLITE_action_replace_NONE('__' + ttline[smmodel.ACTION]))
            if ttline[smmodel.NEXT_STATE].lower() != 'none':  # to not get transitions into/outof state on actions that dont change the state...
                tt_out += ' = ' + even_space(self.transitiontableLITE_nextstate_replace_NONE(ttline[smmodel.NEXT_STATE], ttline[smmodel.START_STATE]))
            tt_out = tt_out.rstrip()
            tt_out += '\n'
            puthere.append(tt_out)
            tt_out = ""
            # State entry/exit, once only
            if not (ttline[smmodel.START_STATE] in startStateHasEntryExit):
                startStateHasEntryExit[ttline[smmodel.START_STATE]] = True
                tt_out += "                , " + ttline[smmodel.START_STATE] + " + msm::on_entry / __" + ttline[smmodel.START_STATE] + 'OnEntry\n'
                tt_out += "                , " + ttline[smmodel.START_STATE] + " + msm::on_exit / __" + ttline[smmodel.START_STATE] + 'OnExit'
                tt_out = tt_out.rstrip()
                tt_out += '\n'
                puthere.append(tt_out)
                tt_out = ""

    def innerexpand_sml(self, smmodel, whitespace, sml_entry_exit, puthere):
        tt_out = whitespace + "// " + even_space("Start", smmodel.maxlenSTART_STATE + 8) + even_space("+Event", smmodel.maxlenEVENT + 10) + even_space("[ Guard ]", smmodel.maxlenGUARD + 6) + even_space("/ Action", smmodel.maxlenACTION + 4) + even_space(" = Next", 0) + '\n'
        startStateHasEntryExit = {}
        for i, ttline in enumerate(smmodel.transition_table):
            if i == 0:  # initial state
                tt_out += whitespace + " *"
            else:
                tt_out += whitespace + ", "
            tt_out += even_space('state<' + self.transitiontable_replace_NONE(ttline[smmodel.START_STATE]) + '>', smmodel.maxlenSTART_STATE + 9) + '+'
            tt_out += even_space('event<' + self.transitiontable_replace_NONE(ttline[smmodel.EVENT]) + '>', smmodel.maxlenEVENT + 9) + ' '
            tt_out += even_space('[' + self.transitiontableLITE_guard_replace_NONE(camel_case_small(ttline[smmodel.GUARD])) + ']', smmodel.maxlenGUARD + 4) + ' / '
            tt_out += even_space(self.transitiontableLITE_action_replace_NONE(camel_case_small(ttline[smmodel.ACTION])), smmodel.maxlenACTION + 2)
            if ttline[smmodel.NEXT_STATE].lower() != 'none':  # to not get transitions into/outof state on actions that dont change the state...
                tt_out += ' = ' + even_space('state<' + self.transitiontableLITE_nextstate_replace_NONE(ttline[smmodel.NEXT_STATE], ttline[smmodel.START_STATE]) + '>', 0)
            tt_out = tt_out.rstrip()
            tt_out += '\n'
            puthere.append(tt_out)
            tt_out = ""
            # State entry/exit, once only
            if not (ttline[smmodel.START_STATE] in startStateHasEntryExit) and sml_entry_exit:
                startStateHasEntryExit[ttline[smmodel.START_STATE]] = True
                tt_out += whitespace + ", state<" + ttline[smmodel.START_STATE] + "> + boost::sml::on_entry<_> / " + camel_case_small(ttline[smmodel.START_STATE]) + 'OnEntry\n'
                tt_out += whitespace + ", state<" + ttline[smmodel.START_STATE] + "> + boost::sml::on_exit<_> / " + camel_case_small(ttline[smmodel.START_STATE]) + 'OnExit'
                tt_out = tt_out.rstrip()
                tt_out += '\n'
                puthere.append(tt_out)
                tt_out = ""

    def innerexpand_transitionsperstate(self, transitionperstate, lines2x, puthere):
        for state, dict in transitionperstate.items():
            ex_transition = False
            snipped_to_expand = []
            for line in lines2x:
                begin = line.find(__TAG_PET_BEGIN__) > -1
                ex_transition = begin  or ex_transition

                if ex_transition and line.find(__TAG_PET_END__) > -1:
                    # ----
                    # Should now have all the Event repeats.
                    for ev, transitionList in dict.items():
                        # guard/action/next state repeats
                        self.innerexpand_transitionsperguard(ev, state, transitionList, snipped_to_expand, puthere)
                    # ----
                    ex_transition = False

                if ex_transition:
                    if not begin:
                        snipped_to_expand.append(line)
                else:
                    if line.find(__TAG_PET_END__) == -1:
                        puthere.append(line.replace(__TAG_STATENAME__, state).replace(__TAG_STATENAME_SMALL_CAMEL__, state))

    def innerexpand_transitionsperguard(self, eventName, stateName, transitionList, lines2x, puthere):
        ex_transition = False
        snipped_to_expand = []
        for line in lines2x:
            begin = line.find(__TAG_PGT_BEGIN__) > -1
            ex_transition = begin or ex_transition
            if ex_transition and line.find(__TAG_PGT_END__) > -1:
                # Should now have all the guard/action/next repeats.
                #----
                for transitionDict in transitionList:
                    for l in snipped_to_expand:
                        for k, v in transitionDict.items():
                            # for those that are present but who have alternate text when not present.
                            if self.hasSpecificTag(l, k):
                                l = self.removeDefault(l)
                            l = l.replace(k, v)
                        l = l.replace(__TAG_EVENTNAME__, eventName)
                        l = l.replace(__TAG_EVENTNAME_SMALL_CAMEL__, camel_case_small(eventName))
                        # If there is no guard, or no next state, or no action, just remove it (or replace it with the alternative text). Leave no hanging code.
                        if self.hasSpecificTag(l,__TAG_GUARDNAME_SMALL_CAMEL__) or self.hasSpecificTag(l,__TAG_GUARDNAME__) or self.hasSpecificTag(l, __TAG_NEXTSTATENAME__) or self.hasSpecificTag(l, __TAG_ACTIONNAME__) or self.hasSpecificTag(l,__TAG_STATENAME_IF_NEXTSTATE__) or self.hasSpecificTag(l, __TAG_STATENAME_IF_NEXTSTATE_SMALL_CAMEL__):
                            line_member = self.extractDefaultAndTag(l)
                            if line_member[1]: # alternative text is embedded in the tag.
                                whitespace = len(l) - len(l.lstrip())
                                puthere.append(whitespace*' ' + line_member[1] + '\n')
                        elif l.find(__TAG_GUARDNAME_SMALL_CAMEL__) == -1 and l.find(__TAG_GUARDNAME__) == -1 and l.find(__TAG_NEXTSTATENAME__) == -1 and l.find(__TAG_STATENAME_IF_NEXTSTATE__) == -1 and l.find(__TAG_STATENAME_IF_NEXTSTATE_SMALL_CAMEL__) == -1:
                            puthere.append(l)
                #----
                ex_transition = False
            if ex_transition:
                if not begin:
                    snipped_to_expand.append(line)
            else:
                if line.find(__TAG_PGT_END__) == -1:
                    puthere.append(line.replace(__TAG_EVENTNAME__, eventName).replace(__TAG_EVENTNAME_SMALL_CAMEL__, camel_case_small(eventName)))

    def transitiontable_replace_NONE(self, val):
        if val == "" or val.lower() == 'none':
            val = "msmf::none"
        return val

    def transitiontableLITE_guard_replace_NONE(self, val):
        tmp_val = val.replace('__', '')
        if tmp_val == "" or tmp_val.lower() == 'none':
            val = "gnone"
        return val

    def transitiontableLITE_action_replace_NONE(self, val):
        tmp_val = val.replace('__', '')
        if tmp_val == "" or tmp_val.lower() == 'none' or tmp_val.lower().find('::none<') > -1:
            val = "none"
        return val

    ''' This SM doesnt seem to allow 'none' transitions -> make it transition to the source state'''
    def transitiontableLITE_nextstate_replace_NONE(self, val, source_state):
        tmp_val = val.replace('__', '')
        tmp_val = tmp_val.replace('msmf::', '')
        if tmp_val == "" or tmp_val.lower() == 'none':
            val = source_state
        return val


    def expand_secondfiltering(self, smmodel, cmmodel):
        for file in cmmodel.filenames_to_lines:

            ex_state       = False
            ex_event       = False
            ex_action      = False
            ex_actionsig   = False
            ex_transition  = False
            ex_guard       = False
            ex_tt          = False
            ex_tt_lite     = False
            ex_tt_lite_sml = False
            sml_entry_exit = False

            snipped_to_expand = []
            alllinesexpanded = []
            for line in cmmodel.filenames_to_lines[file]:
                begin          = line.find(__TAG_PS_BEGIN__) > -1 or \
                                 line.find(__TAG_PE_BEGIN__) > -1 or \
                                 line.find(__TAG_PA_BEGIN__) > -1 or \
                                 line.find(__TAG_PASIG_BEGIN__) > -1 or \
                                 line.find(__TAG_PST_BEGIN__) > -1 or \
                                 line.find(__TAG_PG_BEGIN__) > -1 or \
                                 line.find(__TAG_TTT_BEGIN__) > -1 or \
                                 line.find(__TAG_TTT_LITE_BEGIN__) > -1 or \
                                 line.find(__TAG_TTT_SML_BEGIN__.replace(">","")) > -1

                ex_state       = line.find(__TAG_PS_BEGIN__) > -1 or ex_state
                ex_event       = line.find(__TAG_PE_BEGIN__) > -1 or ex_event
                ex_action      = line.find(__TAG_PA_BEGIN__) > -1 or ex_action
                ex_actionsig   = line.find(__TAG_PASIG_BEGIN__) > -1 or ex_actionsig
                ex_transition  = line.find(__TAG_PST_BEGIN__) > -1 or ex_transition
                ex_guard       = line.find(__TAG_PG_BEGIN__) > -1 or ex_guard
                ex_tt          = line.find(__TAG_TTT_BEGIN__) > -1 or ex_tt
                ex_tt_lite     = line.find(__TAG_TTT_LITE_BEGIN__) > -1 or ex_tt_lite
                ex_tt_lite_sml = line.find(__TAG_TTT_SML_BEGIN__.replace(">","")) > -1 or ex_tt_lite_sml
                sml_entry_exit = line.find(__TAG_TTT_SML_BEGIN_ENTRYEXIT__) > -1 or sml_entry_exit

                if not ex_state and not ex_event and not ex_action and not ex_actionsig and not ex_transition and not ex_guard and not ex_tt and not ex_tt_lite and not ex_tt_lite_sml:
                    alllinesexpanded.append(line.replace(__TAG_INIT_STATE__, smmodel.getfirststate()).replace(__TAG_INIT_STATE_SMALL_CAMEL__, camel_case_small(smmodel.getfirststate())))

                if ex_state and line.find(__TAG_PS_END__) > -1:
                    self.innerexpand_secondfiltering(smmodel.states, snipped_to_expand, alllinesexpanded)
                    snipped_to_expand = []
                    ex_state = False
                if ex_event and line.find(__TAG_PE_END__) > -1:
                    self.innerexpand_secondfiltering(smmodel.events, snipped_to_expand, alllinesexpanded)
                    snipped_to_expand = []
                    ex_event = False
                if ex_action and line.find(__TAG_PA_END__) > -1:
                    self.innerexpand_secondfiltering(smmodel.actions, snipped_to_expand, alllinesexpanded)
                    snipped_to_expand = []
                    ex_action = False
                if ex_actionsig and line.find(__TAG_PASIG_END__) > -1:
                    self.innerexpand_actionsignatures(smmodel.actionsignatures, snipped_to_expand, alllinesexpanded)
                    snipped_to_expand = []
                    ex_actionsig = False
                if ex_transition and line.find(__TAG_PST_END__) > -1:
                    self.innerexpand_transitionsperstate(smmodel.transitionsperstate, snipped_to_expand, alllinesexpanded)
                    snipped_to_expand = []
                    ex_transition = False
                if ex_guard and line.find(__TAG_PG_END__) > -1:
                    self.innerexpand_secondfiltering(smmodel.guards, snipped_to_expand, alllinesexpanded)
                    snipped_to_expand = []
                    ex_guard = False
                if ex_tt and line.find(__TAG_TTT_END___) > -1:
                    self.innerexpand_msm(smmodel, alllinesexpanded)
                    ex_tt = False
                if ex_tt_lite and line.find(__TAG_TTT_LITE_END__) > -1:
                    self.innerexpand_msmlite(smmodel, alllinesexpanded)
                    ex_tt_lite = False
                if ex_tt_lite_sml and line.find(__TAG_TTT_SML_END__) > -1:
                    whitespace = line[0:line.find("<<<")]
                    self.innerexpand_sml(smmodel, whitespace, sml_entry_exit, alllinesexpanded)
                    ex_tt_lite_sml = False

                if (ex_state or ex_event or ex_action or ex_actionsig or ex_transition or ex_guard or ex_tt or ex_tt_lite or ex_tt_lite_sml) and not begin:
                    snipped_to_expand.append(line)

            cmmodel.filenames_to_lines[file] = alllinesexpanded

    ''' Used for State Machine Generation
    '''
    def Generate(self, transitiontable, namespacenname, statemachinename, dclspc="", copyotherfiles = True):

        print("*************************************")
        print("******* SMGen ***********************")
        print("*************************************")
        print(" Output Dir   : " + self.output_gen_file_dir)
        print(" State Machine: " + statemachinename)
        print(" Executing in : " + os.path.realpath(__file__))
        print("*************************************")

        sm = CTransitionTableModel(transitiontable, namespacenname, statemachinename, dclspc)

        for e in self.events_interface.Structs(): # only necessary for stateless (nonTT) events.
            sm.events.add(e.Name)

        cm = self.loadtemplates_firstfiltering(sm)
        self.expand_secondfiltering(sm, cm)

        # user tags.
        if self.events_interface != None:
            self.do_user_tags(cm, self.events_interface.UserTags())

        # Preserve user code.
        self.preserve_usercode_in_files(cm)
        '''
        # Round-trip Code Preservation. Will load the code to preserve upon creation (if the output dir is not-empty/the same as the one in the compile path).
        preservation = Preservative(self.output_gen_file_dir)
        preservation.Emplace(cm.filenames_to_lines)
        '''
        # Write output to file.
        self.createoutput(cm.filenames_to_lines)

        # Copy non-autogenerated required files to output.
        if isinstance(self.language, LanguageCPP) and copyotherfiles:

            # Files...
            files_to_copy = []
            files_to_copy.append("allocator.h")
            files_to_copy.append("allocator.cpp")
            files_to_copy.append("basetypes.h")
            files_to_copy.append("CMakeLists.txt")
            files_to_copy.append("Fault.h")
            files_to_copy.append("Fault.cpp")
            files_to_copy.append("stl_allocator.h")
            files_to_copy.append("thread_FreeRTOS.h")
            files_to_copy.append("thread_FreeRTOS.cpp")
            files_to_copy.append("threaded_dispatcher.h")
            files_to_copy.append("threaded_dispatcher_FreeRTOS.h")
            files_to_copy.append("threadsafe_queue.h")
            files_to_copy.append("threadsafe_queue_FreeRTOS.h")
            files_to_copy.append("waitcondition.h")
            files_to_copy.append("waitcondition.cpp")
            files_to_copy.append("xallocator.h")
            files_to_copy.append("xallocator.cpp")
            files_to_copy.append("xlist.h")
            files_to_copy.append("xmap.h")
            files_to_copy.append("xqueue.h")
            files_to_copy.append("xset.h")
            files_to_copy.append("xsstream.h")
            files_to_copy.append("xstring.h")

            allplatformsfrom = os.path.join(os.path.abspath(os.path.dirname(__file__)), os.path.join("allplatforms", "CPP"))
            allplatformsto = os.path.join(os.path.abspath(self.output_gen_file_dir), "allplatforms")

            FileCopyUtil(allplatformsfrom, allplatformsto, files_to_copy)

            # Boost SML ...
            smlfrom = os.path.join(allplatformsfrom,	os.path.join("sml", os.path.join("include","boost")))
            smlto = os.path.join(allplatformsto, "boost")
            smlfiles_to_copy = []
            smlfiles_to_copy.append("sml.hpp")
            FileCopyUtil(smlfrom, smlto, smlfiles_to_copy)

            # Tests...
            testfiles_to_copy = []
            testfiles_to_copy.append("CMakeLists.txt")
            testfiles_to_copy.append("Test.ThreadingConcepts.cpp")
            testfiles_to_copy.append("test_main.cpp")

            tests_allplatformsfrom = os.path.join(allplatformsfrom, "testsuite")
            tests_allplatformsto = os.path.join(allplatformsto, "testsuite")

            FileCopyUtil(tests_allplatformsfrom, tests_allplatformsto, testfiles_to_copy)

            # Micro Unit Test Framework
            microunit_files_to_copy = []
            microunit_files_to_copy.append("minunit.h")
            microunit_files_to_copy.append("minunit.cpp")

            microunit_allplatformsfrom = os.path.join(tests_allplatformsfrom, "minunit")
            microunit_allplatformsto = os.path.join(tests_allplatformsto, "minunit")

            FileCopyUtil(microunit_allplatformsfrom, microunit_allplatformsto, microunit_files_to_copy)


    ''' Used for Protocol Generation
    '''
    def GenerateProtocol(self, pythoninterfacegeneratorfilename, namespacenname, classname, dclspc="", preserve_dir=""):
        sm = CTransitionTableModel([], namespacenname, classname, dclspc)
        sm.pythoninterfacegeneratorfilename = pythoninterfacegeneratorfilename
        cm = self.loadtemplates_firstfiltering(sm)
        self.expand_secondfiltering(sm, cm)

        # Round-trip Code Preservation. Will load the code to preserve upon creation (if the output dir is not-empty/the same as the one in the compile path).
        # TCP gen might have a different output directory (typically COG will put files into an intermediate dir, and them copy them elsewhere
        preservation = None
        if preserve_dir == "":
            preservation = Preservative(self.output_gen_file_dir)
        else:
            preservation = Preservative(preserve_dir)

        preservation.Emplace(cm.filenames_to_lines)
        # Write output to file.
        self.createoutput(cm.filenames_to_lines)

        # return the filenames
        filenames = []
        for filename in cm.filenames_to_lines.keys():
            filenames.append(filename)
        return filenames

import unittest
from .LanguagePython import LanguagePython
class TestFeatures(unittest.TestCase):

    workingfolder = os.path.join(os.path.dirname(__file__), "test")

    @classmethod
    def setUpClass(cls):
        os.makedirs(cls.workingfolder, exist_ok=True)

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.workingfolder)

    @classmethod
    def create_template_file(cls, list_of_lines):
        assert os.path.isdir(cls.workingfolder)
        with open(os.path.join(cls.workingfolder, "file.t"), 'w') as f:
            for l in list_of_lines:
                f.write("%s\n" % l)

    @classmethod
    def read_lines_of_output_file(cls):
        path = os.path.join(cls.workingfolder, "file.t")
        assert os.path.isfile(path)
        with open(path, 'r') as f:
            return f.readlines()

    @classmethod
    def do_magic(cls, input, interface, tt = []):
        TestFeatures.create_template_file(input)
        smgenerator = CStateMachineGenerator(cls.workingfolder, cls.workingfolder, interface, LanguagePython(), "", "", "")
        smgenerator.Generate(tt, "", "", "", False)
        return TestFeatures.read_lines_of_output_file()

    def test_event_custom_params_nosignature(self):
        input = []
        input.append("<<<PER_EVENT_BEGIN>>>")
        input.append("<<<EVENTSIGNATURE::p1, p2, p3,>>>")
        input.append("<<<PER_EVENT_END>>>")
        s = Struct("somestruct")
        i = Interface('')
        i.AddStruct(s)

        output = TestFeatures.do_magic(input, i)

        self.assertEqual(len(output), 1)
        self.assertEqual(output[0]  , "p1, p2, p3\n")

    def test_event_custom_params_signature(self):
        input = []
        input.append("<<<PER_EVENT_BEGIN>>>")
        input.append("<<<EVENTSIGNATURE::p1, p2, p3,>>>")
        input.append("<<<PER_EVENT_END>>>")
        s = Struct("somestruct")
        s.AddType("binga","bungaBunga")
        i = Interface('')
        i.AddStruct(s)

        output = TestFeatures.do_magic(input, i)

        self.assertEqual(len(output), 1)
        self.assertEqual(output[0], "binga, p1, p2, p3\n") # Python

    def test_event_members_instantiate_custom_name(self):
        input = []
        input.append("<<<PER_EVENT_BEGIN>>>")
        input.append("<<<EVENTMEMBERSLITEINSTANTIATE::hello>>>")
        input.append("<<<PER_EVENT_END>>>")
        s = Struct("somestruct")
        s.AddType("binga", "bungaBunga")
        i = Interface('')
        i.AddStruct(s)

        output = TestFeatures.do_magic(input, i)

        self.assertEqual(len(output), 1)
        self.assertEqual(output[0], "hello.binga = binga\n")

    def test_event_members_instantiate_no_custom_name(self):
        input = []
        input.append("<<<PER_EVENT_BEGIN>>>")
        input.append("<<<EVENTMEMBERSLITEINSTANTIATE>>>")
        input.append("<<<PER_EVENT_END>>>")
        s = Struct("somestruct")
        s.AddType("binga", "bungaBunga")
        i = Interface('')
        i.AddStruct(s)

        output = TestFeatures.do_magic(input, i)

        self.assertEqual(len(output), 1)
        self.assertEqual(output[0], "data.binga = binga\n")

    def test_transitionsperguard_tags(self):
        input = []
        input.append("<<<PER_STATETRANSITION_BEGIN>>>")
        input.append("<<<PER_EVENTTRANSITION_BEGIN>>>")
        input.append("<<<PER_GUARDTRANSITION_BEGIN>>>")
        input.append("<<<GUARDNAME::No Guard>>>")
        #input.append("<<<EVENTNAME::No Event>>>") Hmmm ... need to think more on this.
        input.append("<<<EVENTNAME>>>")
        input.append("<<<STATENAMEIFNEXTSTATE::No Next State>>>")
        input.append("<<<ACTIONNAME::No Action>>>")
        input.append("<<<NEXTSTATENAME::No Next State>>>")
        input.append("<<<PER_GUARDTRANSITION_END>>>")
        input.append("<<<PER_EVENTTRANSITION_END>>>")
        input.append("<<<PER_STATETRANSITION_END>>>")
        tt = [['S1', 'Do', 'S1', 'A1', 'G1']]
        s = Struct("somestruct")
        s.AddType("binga", "bungaBunga")
        i = Interface('')
        i.AddStruct(s)

        output = TestFeatures.do_magic(input, i, tt)

        self.assertEqual(len(output), 5)
        self.assertEqual(output[0], "G1\n")
        self.assertEqual(output[1], "Do\n")
        self.assertEqual(output[2], "S1\n")
        self.assertEqual(output[3], "A1\n")
        self.assertEqual(output[4], "S1\n")

    def test_transitionsperguard_tags_with_custom_defaults(self):
        input = []
        input.append("<<<PER_STATETRANSITION_BEGIN>>>")
        input.append("<<<PER_EVENTTRANSITION_BEGIN>>>")
        input.append("<<<PER_GUARDTRANSITION_BEGIN>>>")
        input.append("<<<GUARDNAME::No Guard>>>")
        input.append("<<<EVENTNAME::No Event>>>")
        input.append("<<<STATENAMEIFNEXTSTATE::No Next State>>>")
        input.append("<<<ACTIONNAME::No Action>>>")
        input.append("<<<NEXTSTATENAME::No Next State>>>")
        input.append("<<<PER_GUARDTRANSITION_END>>>")
        input.append("<<<PER_EVENTTRANSITION_END>>>")
        input.append("<<<PER_STATETRANSITION_END>>>")
        tt = [['S1', 'Do', 'None', 'None', 'None']]
        s = Struct("somestruct")
        s.AddType("binga", "bungaBunga")
        i = Interface('')
        i.AddStruct(s)

        output = TestFeatures.do_magic(input, i, tt)

        self.assertEqual(len(output), 5)
        self.assertEqual(output[0], "No Guard\n")
        self.assertEqual(output[1], "No Event\n")
        self.assertEqual(output[2], "No Next State\n")
        self.assertEqual(output[3], "No Action\n")
        self.assertEqual(output[4], "No Next State\n")

    def test_events(self):
        input = []
        input.append("<<<PER_EVENT_BEGIN>>>")
        input.append("<<<EVENTNAME>>>")
        input.append("<<<PER_EVENT_END>>>")
        tt = [['S1', "how ya doin'", 'None', 'None', 'None']]
        s = Struct("hello")
        i = Interface('')
        i.AddStruct(s)

        output = TestFeatures.do_magic(input, i, tt)

        self.assertEqual(len(output), 2)
        self.assertEqual(output[0], "hello\n")
        self.assertEqual(output[1], "how ya doin'\n")

    '''
    def test_split(self):
        s = 'hello world'
        self.assertEqual(s.split(), ['hello', 'world'])
        # check that s.split fails when the separator is not a string
        with self.assertRaises(TypeError):
            s.split(2)
    '''
if __name__ == '__main__':
    unittest.main()


if __package__ is None:
    import sys
    from os import path
    sys.path.append( path.dirname( path.dirname( path.abspath(__file__) ) ) )


# http://mikegrouchy.com/blog/2012/05/be-pythonic-__init__py.html
#from .smgen import *
__all__ = ['smgen', 'cgen', 'vppclassdiagram', 'vppfs', 'kojentypes', 'umlgen', 'protogen', 'Generate']
